@@ -530,6 +530,8 @@ class DataFrame(metaclass=DataFrameMeta):
 
     def any(self):
         """Returns whether any of the values in the dataframe are truthy. Requires the dataframe to only contain boolean values."""
+        if len(self) == 0:
+            return False
         if not all(isinstance(x, type) and issubclass(x, pl.Boolean) for x in self._underlying.dtypes):
             raise TypeError("DataFrame.any() is not defined on a dataframe that contains non-boolean columns.")
         materialized = self._materialize()
@@ -537,6 +539,8 @@ class DataFrame(metaclass=DataFrameMeta):
 
     def all(self):
         """Returns whether all of the values in the dataframe are truthy. Requires the dataframe to only contain boolean values."""
+        if len(self) == 0:
+            return True
         if not all(isinstance(x, type) and issubclass(x, pl.Boolean) for x in self._underlying.dtypes):
             raise TypeError("DataFrame.any() is not defined on a dataframe that contains non-boolean columns.")
         materialized = self._materialize()
@@ -588,7 +592,7 @@ class DataFrame(metaclass=DataFrameMeta):
 
     def _perform_op(
         self,
-        op: Callable[[Any, Any], Any],
+        op: Union[str, Callable[[Any, Any], Any]],
         other: Union[DataFrame, pl.DataFrame, pd.DataFrame, Any],
         convert_dtypes: bool,
     ):
@@ -599,29 +603,106 @@ class DataFrame(metaclass=DataFrameMeta):
             other = other.collect()
         if isinstance(other, pd.DataFrame):
             other = pl.from_pandas(other)
+        if op in ("eq", "ne", "lt", "le", "ge", "gt"):
+            if isinstance(other, pl.DataFrame):
+                return self._perform_comp_df(op, other, convert_dtypes=convert_dtypes)
+            else:
+                return DataFrame(
+                    getattr(operator, op)(materialized, other),
+                    convert_dtypes=convert_dtypes,
+                    pydantic_model=self._pydantic_model,
+                )
+        assert callable(op)
         return DataFrame(
             op(materialized, other),
             convert_dtypes=convert_dtypes,
             pydantic_model=self._pydantic_model,
         )
 
+    def _perform_comp_df(
+        self,
+        op: str,
+        other: pl.DataFrame,
+        convert_dtypes: bool,
+    ):
+        # There's a bug in the default polars implementation for comparisons -- see
+        # https://github.com/pola-rs/polars/issues/5870
+        materialized = self._materialize()
+        if materialized.columns != other.columns:
+            raise ValueError("DataFrame columns do not match")
+        if materialized.shape != other.shape:
+            raise ValueError("DataFrame dimensions do not match")
+
+        suffix = "__POLARS_CMP_OTHER"
+        other_renamed = other.select(pl.all().suffix(suffix))
+        combined = pl.concat([materialized, other_renamed], how="horizontal")
+
+        if op == "eq":
+            expr = [
+                (
+                    pl.when(pl.col(n).is_null() & pl.col(f"{n}{suffix}").is_null())
+                    .then(pl.lit(True))
+                    .otherwise(pl.col(n) == pl.col(f"{n}{suffix}"))
+                    .alias(n)
+                )
+                for n in materialized.columns
+            ]
+        elif op == "ne":
+            expr = [
+                (
+                    pl.when(pl.col(n).is_null() & pl.col(f"{n}{suffix}").is_null())
+                    .then(pl.lit(False))
+                    .otherwise(pl.col(n) != pl.col(f"{n}{suffix}"))
+                    .alias(n)
+                )
+                for n in materialized.columns
+            ]
+        elif op == "gt":
+            expr = [pl.col(n) > pl.col(f"{n}{suffix}") for n in materialized.columns]
+        elif op == "lt":
+            expr = [pl.col(n) < pl.col(f"{n}{suffix}") for n in materialized.columns]
+        elif op == "ge":
+            expr = [
+                (
+                    pl.when(pl.col(n).is_null() & pl.col(f"{n}{suffix}").is_null())
+                    .then(pl.lit(True))
+                    .otherwise(pl.col(n) >= pl.col(f"{n}{suffix}"))
+                    .alias(n)
+                )
+                for n in materialized.columns
+            ]
+        elif op == "le":
+            expr = [
+                (
+                    pl.when(pl.col(n).is_null() & pl.col(f"{n}{suffix}").is_null())
+                    .then(pl.lit(True))
+                    .otherwise(pl.col(n) <= pl.col(f"{n}{suffix}"))
+                    .alias(n)
+                )
+                for n in materialized.columns
+            ]
+        else:
+            raise ValueError(f"got unexpected comparison operator: {op}")
+
+        return DataFrame(combined.select(expr), convert_dtypes=convert_dtypes, pydantic_model=self._pydantic_model)
+
     def __eq__(self, other: Union[DataFrame, pl.DataFrame, pl.LazyFrame, pd.DataFrame, Any]):  # type: ignore
-        return self._perform_op(operator.eq, other, convert_dtypes=False)
+        return self._perform_op("eq", other, convert_dtypes=False)
 
     def __ne__(self, other: Union[DataFrame, pl.DataFrame, pl.LazyFrame, pd.DataFrame, Any]):  # type: ignore
-        return self._perform_op(operator.ne, other, convert_dtypes=False)
+        return self._perform_op("ne", other, convert_dtypes=False)
 
     def __gt__(self, other: Union[DataFrame, pl.DataFrame, pl.LazyFrame, pd.DataFrame, Any]):
-        return self._perform_op(operator.gt, other, convert_dtypes=False)
+        return self._perform_op("gt", other, convert_dtypes=False)
 
     def __lt__(self, other: Union[DataFrame, pl.DataFrame, pl.LazyFrame, pd.DataFrame, Any]):
-        return self._perform_op(operator.lt, other, convert_dtypes=False)
+        return self._perform_op("lt", other, convert_dtypes=False)
 
     def __ge__(self, other: Union[DataFrame, pl.DataFrame, pl.LazyFrame, pd.DataFrame, Any]):
-        return self._perform_op(operator.ge, other, convert_dtypes=False)
+        return self._perform_op("ge", other, convert_dtypes=False)
 
     def __le__(self, other: Union[DataFrame, pl.DataFrame, pl.LazyFrame, pd.DataFrame, Any]):
-        return self._perform_op(operator.le, other, convert_dtypes=False)
+        return self._perform_op("le", other, convert_dtypes=False)
 
     def __add__(self, other: Union[DataFrame, pl.DataFrame, pl.LazyFrame, pd.DataFrame, Any]) -> DataFrame:
         return self._perform_op(operator.add, other, convert_dtypes=True)
@@ -669,6 +750,44 @@ class DataFrame(metaclass=DataFrameMeta):
         pd_dataframe = pd_dataframe.rename(columns={x.root_fqn: x for x in self.columns})
         return pd_dataframe
 
+    def to_features(self) -> Sequence[Features]:
+        """Get values in the dataframe as :class:`Features` instances."""
+        from chalk.features.feature_set import FeatureSetBase
+
+        if self.namespace is None:
+            raise ValueError("This method is not supported if the DataFrame spans multiple namespaces")
+        ans: List[Features] = []
+        for row in self._materialize().to_dicts():
+            rooted_prefix_to_values: Dict[str, Dict[str, Any]] = {}
+
+            for k, v in row.items():
+                rooted_prefix_split = k.split(".")[:-1]
+                for i in range(1, len(rooted_prefix_split) + 1):
+                    rooted_prefix = ".".join(rooted_prefix_split[:i])
+                    if rooted_prefix not in rooted_prefix_to_values:
+                        rooted_prefix_to_values[rooted_prefix] = {}
+                rooted_prefix_to_values[".".join(rooted_prefix_split)][k] = v
+            # Sorting in reverse to construct the innermost features first
+            sorted_sub_features = sorted(rooted_prefix_to_values.keys(), key=lambda x: len(x), reverse=True)
+
+            for rooted_prefix in sorted_sub_features:
+                values = rooted_prefix_to_values[rooted_prefix]
+                sub_kwargs: Dict[str, Any] = {}
+                for k, v in values.items():
+                    feature = Feature.from_root_fqn(k)
+                    sub_kwargs[feature.attribute_name] = FEATURE_CODEC.decode(feature, v)
+                if rooted_prefix == self.namespace:
+                    features_cls = FeatureSetBase.registry[self.namespace]
+                    ans.append(features_cls(**sub_kwargs))
+                else:
+                    rooted_prefix_split = rooted_prefix.split(".")
+                    features_cls = Feature.from_root_fqn(rooted_prefix).joined_class
+                    assert features_cls is not None
+                    parent_rooted_prefix, feature_name = ".".join(rooted_prefix_split[:-1]), rooted_prefix_split[-1]
+                    del feature_name  # unused
+                    rooted_prefix_to_values[parent_rooted_prefix][rooted_prefix] = features_cls(**sub_kwargs)
+        return ans
+
     #######################
     # Filtering / Selecting
     #######################
@@ -722,6 +841,15 @@ class DataFrame(metaclass=DataFrameMeta):
             df = materialized[item]
             return DataFrame(df, pydantic_model=self._pydantic_model)
 
+    def slice(self, offset: int = 0, length: Optional[int] = None):
+        """Slice the dataframe
+
+        Args:
+            offset: The offset to start at.
+            length: The number of rows in the slice. If None (the default), include all rows from ``offset`` to the end of the DataFrame.
+        """
+        return DataFrame(self._underlying.slice(offset, length))
+
 
 def _feature_type_or_value(e: Union[Feature, FeatureWrapper]):
     if isinstance(e, FeatureWrapper):
